﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.Net;
using System.Web;
using DroidExplorer.Core;
using System.IO;
using DroidExplorer.Core.Adb;

namespace DroidExplorer.Configuration.Net {
	public class CloudStatistics {
		
		private const String BASE_URL = "http://deapi.twimager.com/";

		private CloudStatistics ( ) {

		}

		private static CloudStatistics _instance = null;
		public static CloudStatistics Instance {
			get {
				if ( _instance == null ) {
					_instance = new CloudStatistics ( );
				}
				return _instance;
			}
		}

		private String[] FilteredProperties {
			get {
				return new String[] {
					"persist.radio.imei",
					"ril.IMEI",
					"ril.IMSI",
					"ril.iccid",
					"ril.serialnumber",
					"ro.boot.serialno",
					"ro.gsm.imei",
					"ro.ril.MEID",
					"ro.ril.MDN",
					"persist.radio.vzw.cdma.mdn"
				};
			}
		}

		public void RegisterDevice ( Device device ) {
			try {

				// the service is down for now so i am turning this off until i get it running again
				return;

				// ID is an MD5 hash
				/*MD5 md5 = new MD5CryptoServiceProvider ( );
				var hash = md5.ComputeHash ( device.SerialNumber.ToByteArray ( ) );
				String id = hash.ToBase64String ( );*/
				var kvp = new List<KeyValuePair<String, String>> ( );
				var useragent = String.Format ( "{0}", this.GetType ( ).Assembly.GetName ( ).Version.ToString ( ) );
				kvp.Add ( new KeyValuePair<string, string> ( "Id", device.SerialNumber ) );
				kvp.Add ( new KeyValuePair<string, string> ( "UserAgent", useragent ) );

				// current version of droid explorer requires root and busybox so these are hard coded.
				kvp.Add ( new KeyValuePair<string, string> ( "HasBusyBox", true.ToString() ) );
				kvp.Add ( new KeyValuePair<string, string> ( "HasRoot", true.ToString() ) );

				device.Properties.Add ( "ro.droidexplorer.version", useragent );
				device.Properties.Add ( "ro.droidexplorer.root", true.ToString() );
				device.Properties.Add ( "ro.droidexplorer.busybox", true.ToString() );
				device.Properties.Add ( "ro.droidexplorer.architecture", Architecture.IsRunningX64 ? "x64" : "x86" );
				device.Properties.Add ( "ro.droidexplorer.platform", Environment.OSVersion.Platform.ToString ( ) );
				device.Properties.Add ( "ro.droidexplorer.platformversion", Environment.OSVersion.VersionString );

				var propCount = 0;
				foreach ( var item in device.Properties ) {
					// check if the property is filtered. we dont want to pass some info, like IMEI.
					if ( !FilteredProperties.Contains ( item.Key ) ) {
						kvp.Add ( new KeyValuePair<String, String> ( String.Format ( "Properties[{0}].Name", propCount ), item.Key ) );
						kvp.Add ( new KeyValuePair<String, String> ( String.Format ( "Properties[{0}].Value", propCount ), item.Value ) );
						kvp.Add ( new KeyValuePair<String, String> ( String.Format ( "Properties[{0}].DeviceId", propCount ), device.SerialNumber ) );
						++propCount;
					}
				}

				var req = HttpWebRequest.Create ( CreateUrl ( "api/device/add" ) ) as HttpWebRequest;
				req.UserAgent = useragent;
				req.ContentType = "application/x-www-form-urlencoded";
				req.Method = "POST";

				var data = CreateFormattedPostRequest ( kvp );

				var bytes = data.GetBytes ( );
				req.ContentLength = bytes.Length;
				req.Timeout = 30 * 1000;
				using ( var rs = req.GetRequestStream ( ) ) {
					rs.Write ( bytes, 0, bytes.Length );
				}
				using ( var resp = req.GetResponse ( ) as HttpWebResponse ) {
					if ( resp.StatusCode != HttpStatusCode.OK ) {
						this.LogError ( String.Format ( "POST Statistics Failed (Error Code: {1}): {0}", resp.StatusDescription, resp.StatusCode ) );
					}
				}
			} catch ( WebException ex ) {
				this.LogDebug ( "Url: {0}", CreateUrl ( "api/device/add" ) );
				this.LogError ( ex.Message, ex );
			}
		}

		private String CreateUrl ( String path ) {
			var ub = new Uri ( BASE_URL );
			var u = new Uri ( ub, path );

			return u.ToString ( );
		}

		private String CreateFormattedPostRequest ( ICollection<KeyValuePair<String, String>> values ) {
			var paramterBuilder = new StringBuilder ( );
			var counter = 0;
			foreach ( var value in values ) {
				paramterBuilder.AppendFormat ( "{0}={1}", value.Key, HttpUtility.UrlEncode ( value.Value ) );
				if ( counter != values.Count - 1 ) {
					paramterBuilder.Append ( "&" );
				}
				counter++;
			}

			return paramterBuilder.ToString ( );
		}
	}
}
