﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using DroidExplorer.Core;
using Microsoft.Win32;
using DroidExplorer.Core.Configuration;

namespace DroidExplorer.Configuration.UI {
	public class SdkUIEditor : Control, IUIEditor {
		private delegate void SetControlTextDelegate ( Control ctrl, string text );

		private Label label1;
		private Button browse;
		private TextBox sdkPath;
		private GroupBox groupBox1;
		private Label aaptVersion;
		private Label label4;
		private Label adbVersion;
		private Label label2;
		private Label label3;
		private Label driverVersion;

		private string usbDriverVersionString;

		/// <summary>
		/// Initializes a new instance of the <see cref="SdkUIEditor"/> class.
		/// </summary>
		public SdkUIEditor ( ) {
			InitializeComponent ( );
			sdkPath.Text = Settings.Instance.SystemSettings.SdkPath;
			adbVersion.Text = CommandRunner.Instance.GetAdbVersion ( );
			aaptVersion.Text = CommandRunner.Instance.GetAaptVersion ( );
			driverVersion.Text = "Loading...";
		}

		#region IUIEditor Members

		/// <summary>
		/// Sets the source object.
		/// </summary>
		/// <param name="obj">The obj.</param>
		public void SetSourceObject ( object obj ) {
			usbDriverVersionString = obj != null ? obj.ToString() : string.Empty;
			if ( this.InvokeRequired ) {
				this.Invoke ( new SetControlTextDelegate ( SetControlText ), driverVersion, usbDriverVersionString );
			} else {
				SetControlText ( driverVersion, usbDriverVersionString );
			}
		}

		#endregion

		/// <summary>
		/// Sets the control text.
		/// </summary>
		/// <param name="ctrl">The CTRL.</param>
		/// <param name="text">The text.</param>
		private void SetControlText ( Control ctrl, string text ) {
			ctrl.Text = text;
		}

		/// <summary>
		/// Initializes the component.
		/// </summary>
		private void InitializeComponent ( ) {
			this.label1 = new System.Windows.Forms.Label ( );
			this.browse = new System.Windows.Forms.Button ( );
			this.sdkPath = new System.Windows.Forms.TextBox ( );
			this.groupBox1 = new System.Windows.Forms.GroupBox ( );
			this.aaptVersion = new System.Windows.Forms.Label ( );
			this.label4 = new System.Windows.Forms.Label ( );
			this.adbVersion = new System.Windows.Forms.Label ( );
			this.label2 = new System.Windows.Forms.Label ( );
			this.label3 = new System.Windows.Forms.Label ( );
			this.driverVersion = new System.Windows.Forms.Label ( );
			this.groupBox1.SuspendLayout ( );
			this.SuspendLayout ( );
			// 
			// label1
			// 
			this.label1.AutoSize = true;
			this.label1.Location = new System.Drawing.Point ( 13, 10 );
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size ( 96, 13 );
			this.label1.TabIndex = 2;
			this.label1.Text = "Android SDK Path:";
			// 
			// browse
			// 
			this.browse.Location = new System.Drawing.Point ( 323, 25 );
			this.browse.Name = "browse";
			this.browse.Size = new System.Drawing.Size ( 75, 23 );
			this.browse.TabIndex = 1;
			this.browse.Text = "&Browse";
			this.browse.Click += new EventHandler ( browse_Click );
			this.browse.UseVisualStyleBackColor = true;
			this.browse.Visible = true;
			// 
			// sdkPath
			// 
			this.sdkPath.Location = new System.Drawing.Point ( 13, 27 );
			this.sdkPath.Name = "sdkPath";
			this.sdkPath.ReadOnly = true;
			this.sdkPath.Size = new System.Drawing.Size ( /*379*/ 305, 20 );
			this.sdkPath.TabIndex = 0;
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add ( this.driverVersion );
			this.groupBox1.Controls.Add ( this.label3 );
			this.groupBox1.Controls.Add ( this.aaptVersion );
			this.groupBox1.Controls.Add ( this.label4 );
			this.groupBox1.Controls.Add ( this.adbVersion );
			this.groupBox1.Controls.Add ( this.label2 );
			this.groupBox1.Location = new System.Drawing.Point ( 13, 87 );
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size ( 385, 128 );
			this.groupBox1.TabIndex = 4;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "SDK Tools Information";
			// 
			// aaptVersion
			// 
			this.aaptVersion.Location = new System.Drawing.Point ( 279, 51 );
			this.aaptVersion.Name = "aaptVersion";
			this.aaptVersion.Size = new System.Drawing.Size ( 100, 13 );
			this.aaptVersion.TabIndex = 6;
			this.aaptVersion.Text = "0.0.0.0";
			this.aaptVersion.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// label4
			// 
			this.label4.AutoSize = true;
			this.label4.Location = new System.Drawing.Point ( 6, 51 );
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size ( 191, 13 );
			this.label4.TabIndex = 5;
			this.label4.Text = "Android Asset Packaging Tool Version:";
			// 
			// adbVersion
			// 
			this.adbVersion.Location = new System.Drawing.Point ( 279, 25 );
			this.adbVersion.Name = "adbVersion";
			this.adbVersion.Size = new System.Drawing.Size ( 100, 13 );
			this.adbVersion.TabIndex = 4;
			this.adbVersion.Text = "0.0.0.0";
			this.adbVersion.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// label2
			// 
			this.label2.AutoSize = true;
			this.label2.Location = new System.Drawing.Point ( 6, 25 );
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size ( 152, 13 );
			this.label2.TabIndex = 3;
			this.label2.Text = "Android Debug Bridge Version:";

			// 
			// label3
			// 
			this.label3.AutoSize = true;
			this.label3.Location = new System.Drawing.Point ( 6, 78 );
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size ( 101, 13 );
			this.label3.TabIndex = 7;
			this.label3.Text = "USB Driver Version:";
			// 
			// driverVersion
			// 
			this.driverVersion.Location = new System.Drawing.Point ( 258, 78 );
			this.driverVersion.Name = "driverVersion";
			this.driverVersion.Size = new System.Drawing.Size ( 121, 13 );
			this.driverVersion.TabIndex = 8;
			this.driverVersion.Text = "{0} ({1})";
			this.driverVersion.TextAlign = System.Drawing.ContentAlignment.MiddleRight;

			this.Controls.Add ( this.label1 );
			this.Controls.Add ( this.browse );
			this.Controls.Add ( this.sdkPath );
			this.Controls.Add ( this.groupBox1 );
			this.Dock = DockStyle.Fill;
			
			this.groupBox1.ResumeLayout ( false );
			this.groupBox1.PerformLayout ( );
			this.ResumeLayout ( false );

		}

		/// <summary>
		/// Handles the Click event of the browse control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		void browse_Click ( object sender, EventArgs e ) {
			FolderBrowserDialog fbd = new FolderBrowserDialog ( );
			fbd.Description = "Select Root of Android SDK Directory";
			if ( fbd.ShowDialog ( this.FindForm ( ) ) == DialogResult.OK ) {
				string dir = fbd.SelectedPath;
				if ( VerifySdkPath ( dir ) ) {
					this.sdkPath.Text = dir;
					Settings.Instance.SystemSettings.SdkPath = this.sdkPath.Text;
					CommandRunner.Instance.SdkPath = Settings.Instance.SystemSettings.SdkPath;
					Settings.Instance.SystemSettings.UseExistingSdk = true;
					adbVersion.Text = CommandRunner.Instance.GetAdbVersion ( );
					aaptVersion.Text = CommandRunner.Instance.GetAaptVersion ( );
				} else {
					MessageBox.Show ( "Unable to locate required SDK tools in the selected directory. Make sure you are selecting the ROOT of the SDK directory.", "Error Locating SDK Tools", MessageBoxButtons.OK, MessageBoxIcon.Error );
				}
			}
		}

		private bool VerifySdkPath ( String path ) {
			DirectoryInfo sdk = new DirectoryInfo ( path );
			if ( !sdk.Exists ) {
				return false;
			} else {
				DirectoryInfo tools = new DirectoryInfo ( Path.Combine ( sdk.FullName, "tools" ) );
				if ( !tools.Exists ) {
					return false;
				}

				FileInfo adb = new FileInfo ( Path.Combine ( tools.FullName, "adb.exe" ) );
				if ( !adb.Exists ) {
					return false;
				}
				return GetLatestSdkPlatform ( path );
			}
		}

		public bool GetLatestSdkPlatform ( String path ) {
			DirectoryInfo sdk = new DirectoryInfo ( path );
			if ( !sdk.Exists ) {
				return false;
			} else {
				DirectoryInfo platforms = new DirectoryInfo ( Path.Combine ( path, "platforms" ) );
				String platformBaseName = "android-";
				List<String> versions = new List<String> ( );
				if ( !platforms.Exists ) {
					return false;
				}
				foreach ( var item in platforms.GetDirectories ( platformBaseName + "*" ) ) {
					try {
						String version = item.Name.Substring ( platformBaseName.Length );
						this.LogDebug ( "Found platform version {0}", version );
						versions.Add ( version );
					} catch ( Exception ex ) {
						this.LogError ( ex.Message, ex );
					}
				}

				if ( versions.Count == 0 ) {
					return false;
				}

				versions.Sort ( );

				try {
					using ( RegistryKey key = Registry.CurrentUser.CreateSubKey ( String.Format(@"{0}\InstallPath",RegistrySettings.SETTINGS_KEY ) ) ) {
						if ( key != null ) {
							this.LogDebug ( "Setting platform data: {0}", versions[versions.Count - 1] );
							key.SetValue ( "Platform", versions[versions.Count - 1] );
							return true;
						}
					}
				} catch ( UnauthorizedAccessException ex ) {
					this.LogWarn ( "Unable to set platform version", ex );
				}

				this.LogWarn ( "Unable to set platform version" );
				return false;
			}
		}
	}
}
