﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace DroidExplorer.Core.UI {
  public class SystemImageListHost {
    public const string APK_IMAGE_CACHE = @"assets\apk";

    private static SystemImageListHost _instance = null;
    private Dictionary<string, int> _systemIcons = null;
    private ImageList _smallImageList = null;
    private ImageList _largeImageList = null;


    private SystemImageListHost ( ) {
      SystemIcons = new Dictionary<string, int> ( );
      SmallSystemImageList = new SystemImageList ( SystemImageListSize.SmallIcons );
      LargeSystemImageList = new SystemImageList ( SystemImageListSize.LargeIcons );
    }

    public SystemImageList SmallSystemImageList { get; private set; }
    public SystemImageList LargeSystemImageList { get; private set; }

    public ImageList SmallImageList {
      get {
        if ( _smallImageList == null ) {
          _smallImageList = new ImageList ( );
          _smallImageList.ImageSize = new Size ( 16, 16 );
          _smallImageList.ColorDepth = ColorDepth.Depth32Bit;
          LoadCachedApkIconsToImageList ( _smallImageList );
        }
        return _smallImageList;
      }
    }

    public ImageList LargeImageList {
      get {
        if ( _largeImageList == null ) {
          _largeImageList = new ImageList ( );
          _largeImageList.ImageSize = new Size ( 32, 32 );
          _largeImageList.ColorDepth = ColorDepth.Depth32Bit;
          LoadCachedApkIconsToImageList ( _largeImageList );
        }
        return _largeImageList;
      }
    }

    public Dictionary<string, int> SystemIcons {
      get;
      private set;
    }

    public void AddFileTypeImage ( string ext, Image smallImage, Image largeImage ) {
      if ( SystemIcons.ContainsKey ( ext ) ) {
        SystemIcons.Remove ( ext );
      }

      SystemIcons.Add ( ext, SmallImageList.Images.Count );
      AlphaImageList.AddFromImage ( smallImage, SmallImageList );
      AlphaImageList.AddFromImage ( largeImage, LargeImageList );
    }


    private void LoadCachedApkIconsToImageList ( ImageList il ) {
      AlphaImageList.AddFromImage ( ( Image )Properties.Resources.folderClosed, SmallImageList );
      AlphaImageList.AddFromImage ( ( Image )Properties.Resources.folderClosedSL, SmallImageList );
      AlphaImageList.AddFromImage ( ( Image )Properties.Resources.txt, SmallImageList );
      AlphaImageList.AddFromImage ( ( Image )Properties.Resources.exec, SmallImageList );
      AlphaImageList.AddFromImage ( ( Image )Properties.Resources.txtSL, SmallImageList );
      AlphaImageList.AddFromImage ( ( Image )Properties.Resources.execSL, SmallImageList );


      AlphaImageList.AddFromImage ( ( Image )Properties.Resources.FolderClosed32, LargeImageList );
      AlphaImageList.AddFromImage ( ( Image )Properties.Resources.FolderClosedSL32, LargeImageList );
      AlphaImageList.AddFromImage ( ( Image )Properties.Resources.GenericDocument32, LargeImageList );
      AlphaImageList.AddFromImage ( ( Image )Properties.Resources.Gear32, LargeImageList );
      AlphaImageList.AddFromImage ( ( Image )Properties.Resources.GenericDocumentSL32, LargeImageList );
      AlphaImageList.AddFromImage ( ( Image )Properties.Resources.GearSL32, LargeImageList );

      AddFileTypeImage ( ".txt", ( Image )Properties.Resources.text, ( Image )Properties.Resources.text32 );
      AddFileTypeImage ( ".csv", ( Image )Properties.Resources.text, ( Image )Properties.Resources.text32 );
      AddFileTypeImage ( ".so", ( Image )Properties.Resources.library, ( Image )Properties.Resources.library32 );
      AddFileTypeImage ( ".dex", ( Image )Properties.Resources.package, ( Image )Properties.Resources.package32 );
      AddFileTypeImage ( ".odex", ( Image )Properties.Resources.package, ( Image )Properties.Resources.package32 );
      AddFileTypeImage ( ".sqlite", ( Image )Properties.Resources.database, ( Image )Properties.Resources.database32 );
      AddFileTypeImage ( ".db", ( Image )Properties.Resources.database, ( Image )Properties.Resources.database32 );
      AddFileTypeImage ( ".database", ( Image )Properties.Resources.database, ( Image )Properties.Resources.database32 );


      string path = System.IO.Path.Combine ( CommandRunner.Settings.UserDataDirectory, APK_IMAGE_CACHE );
      System.IO.DirectoryInfo apkCache = new System.IO.DirectoryInfo ( path );
      if ( !apkCache.Exists ) {
        apkCache.Create ( );
      }

      foreach ( System.IO.FileInfo fi in apkCache.GetFiles ( "*.png" ) ) {
        Image image = Image.FromFile ( fi.FullName );
        AddFileTypeImage ( System.IO.Path.GetFileNameWithoutExtension ( fi.Name ).ToLower ( ), image, image );
      }
    }

    #region static methods / properties
    public static SystemImageListHost Instance {
      get {
        if ( _instance == null ) {
          _instance = new SystemImageListHost ( );
        }
        return _instance;
      }
    }
    #endregion
  }
}
