﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DroidExplorer.Core {
  public class LinuxConsole {
    public const string COLOR_START = @"\e[";
    public const string COLOR_STOP = @"\e[m";
    public const string COLOR_CODE_END = "m";

    public enum ConsoleColor {
      DEFAULT = 0,
      Black = 30,
      Red = 31,
      Green = 32,
      Brown = 33,
      Blue = 34,
      Purple = 35,
      Cyan = 36
    }

    public enum ConsoleColorAttribute {
      Normal = 0,
      Light = 1
    }

    public static ConsoleColor GetConsoleColor ( string code ) {
      string[] data = code.Split ( new char[ ] { ';' } );
      if ( data.Length != 2 ) {
        return ConsoleColor.DEFAULT;
      }
      int i = 0;
      int.TryParse ( data[ 1 ], out i );
      try {
        return ( ConsoleColor )i;
      } catch {
        return ConsoleColor.DEFAULT;
      }
    }

    public static ConsoleColorAttribute GetConsoleColorAttribute ( string code ) {
      string[] data = code.Split ( new char[ ] { ';' } );
      if ( data.Length == 0 ) {
        return ConsoleColorAttribute.Normal;
      }
      int i = 0;
      int.TryParse ( data[ 0 ], out i );
      try {
        return ( ConsoleColorAttribute )i;
      } catch {
        return ConsoleColorAttribute.Light;
      }
    }

    public static System.ConsoleColor ToWindowsConsoleColor ( ConsoleColor color, ConsoleColorAttribute attrib ) {
      switch ( color ) {
        case ConsoleColor.Black:
          return System.ConsoleColor.Black;
        case ConsoleColor.Red:
          if ( attrib != ConsoleColorAttribute.Light ) {
            return System.ConsoleColor.DarkRed;
          } else {
            return System.ConsoleColor.Red;
          }
        case ConsoleColor.Green:
          if ( attrib != ConsoleColorAttribute.Light ) {
            return System.ConsoleColor.DarkGreen;
          } else {
            return System.ConsoleColor.Green;
          }
        case ConsoleColor.Brown:
          if ( attrib != ConsoleColorAttribute.Light ) {
            return System.ConsoleColor.DarkGray;
          } else {
            return System.ConsoleColor.Gray;
          }
        case ConsoleColor.Blue:
          if ( attrib != ConsoleColorAttribute.Light ) {
            return System.ConsoleColor.DarkBlue;
          } else {
            return System.ConsoleColor.Blue;
          }
        case ConsoleColor.Purple:
          if ( attrib != ConsoleColorAttribute.Light ) {
            return System.ConsoleColor.DarkMagenta;
          } else {
            return System.ConsoleColor.Magenta;
          }
        case ConsoleColor.Cyan:
          if ( attrib != ConsoleColorAttribute.Light ) {
            return System.ConsoleColor.DarkCyan;
          } else {
            return System.ConsoleColor.Cyan;
          }
      }

      return System.ConsoleColor.DarkGreen;
    }

  }
}
