﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DroidExplorer.Core;
using DroidExplorer.Core.Plugins;
using System.Windows.Forms;
using DroidExplorer.Plugins.UI;

namespace DroidExplorer.Plugins {
	/// <summary>
	/// 
	/// </summary>
	public class Bart : IPaidPluginExtendedInfo {

		private const string SIGNATURE = "24b2c3d08e17eb615ef970e04b704165";
		private const string BARTAPK = "/data/app/com.droidexplorer.bart.apk";
		private const string BARTAPK1 = "/data/app/com.droidexplorer.bart-1.apk";
		private const string BARTALTAPK = "/sd-ext/app/com.droidexplorer.bart.apk";
		private const string BARTALTAPK1 = "/sd-ext/app/com.droidexplorer.bart-1.apk";

		/// <summary>
		/// Initializes a new instance of the <see cref="Bart"/> class.
		/// </summary>
		/// <param name="host">The host.</param>
		public Bart ( IPluginHost host ) {
			PluginHost = host;
			CommandRunner.Instance.DeviceStateChanged += new EventHandler<DeviceEventArgs> ( CommandRunner_DeviceStateChanged );
		}

		/// <summary>
		/// Handles the DeviceStateChanged event of the CommandRunner control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="DroidExplorer.Core.DeviceEventArgs"/> instance containing the event data.</param>
		void CommandRunner_DeviceStateChanged ( object sender, DeviceEventArgs e ) {
			if ( e.Device.CompareTo ( CommandRunner.Instance.DefaultDevice ) == 0 ) {
				DeviceState = e.State;
				/*if ( ToolStripMenuItem != null ) {
					ToolStripMenuItem.SetEnabled ( enabled );
				}
				if ( ToolStripButton != null ) {
					ToolStripButton.SetEnabled ( enabled );
				}*/

			}
		}

		private CommandRunner.DeviceState DeviceState { get; set; }

		#region IPluginExtendedInfo Members

		/// <summary>
		/// Gets the author.
		/// </summary>
		/// <value>The author.</value>
		public string Author {
			get { return "Ryan Conrad"; }
		}

		/// <summary>
		/// Gets the URL.
		/// </summary>
		/// <value>The URL.</value>
		public string Url {
			get { return "http://forum.xda-developers.com/showthread.php?t=562292"; }
		}

		/// <summary>
		/// Gets the contact.
		/// </summary>
		/// <value>The contact.</value>
		public string Contact {
			get { return string.Empty; }
		}

		/// <summary>
		/// Gets the copyright.
		/// </summary>
		/// <value>The copyright.</value>
		public string Copyright {
			get { return "Copyright © Ryan Conrad 2009"; }
		}

		#endregion

		#region IPlugin Members

		/// <summary>
		/// Gets the name.
		/// </summary>
		/// <value>The name.</value>
		public string Name {
			get { return "Bart Manager"; }
		}

		/// <summary>
		/// Gets the description.
		/// </summary>
		/// <value>The description.</value>
		public string Description {
			get { return "Backup and Restore Tool for android"; }
		}

		/// <summary>
		/// Gets a value indicating whether this instance has configuration.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if this instance has configuration; otherwise, <c>false</c>.
		/// </value>
		public bool HasConfiguration {
			get { return false; }
		}

		/// <summary>
		/// Gets the image.
		/// </summary>
		/// <value>The image.</value>
		public System.Drawing.Image Image {
			get { return Properties.Resources.Bart; }
		}

		/// <summary>
		/// Gets the text.
		/// </summary>
		/// <value>The text.</value>
		public string Text {
			get { return "Bart Manager"; }
		}

		/// <summary>
		/// Gets a value indicating whether [create tool button].
		/// </summary>
		/// <value><c>true</c> if [create tool button]; otherwise, <c>false</c>.</value>
		public bool CreateToolButton {
			get { return true; }
		}

		/// <summary>
		/// Gets the display style.
		/// </summary>
		/// <value>The display style.</value>
		public System.Windows.Forms.ToolStripItemDisplayStyle DisplayStyle {
			get { return System.Windows.Forms.ToolStripItemDisplayStyle.Image; }
		}

		/// <summary>
		/// Executes the specified plugin host.
		/// </summary>
		/// <param name="pluginHost">The plugin host.</param>
		/// <param name="currentDirectory">The current directory.</param>
		public void Execute ( IPluginHost pluginHost, DroidExplorer.Core.IO.LinuxDirectoryInfo currentDirectory ) {
			Execute ( pluginHost, currentDirectory, null );
		}

		/// <summary>
		/// Executes the specified plugin host.
		/// </summary>
		/// <param name="pluginHost">The plugin host.</param>
		public void Execute ( IPluginHost pluginHost ) {
			Execute ( pluginHost, new DroidExplorer.Core.IO.LinuxDirectoryInfo ( "/" ), null );
		}

		/// <summary>
		/// Executes the specified plugin host.
		/// </summary>
		/// <param name="pluginHost">The plugin host.</param>
		/// <param name="currentDirectory">The current directory.</param>
		/// <param name="args">The args.</param>
		public void Execute ( IPluginHost pluginHost, DroidExplorer.Core.IO.LinuxDirectoryInfo currentDirectory, string[] args ) {
			if ( !IsSignatureMatch ) {

				if ( pluginHost == null ) {
					new BartPurchaseDialog ( ).ShowDialog ( null );
				} else {
					new BartPurchaseDialog ( ).ShowDialog ( pluginHost.GetHostWindow ( ) );
				}

				return;
			}

			if ( DeviceState == CommandRunner.DeviceState.Recovery ) {
				if ( BartDialog == null || BartDialog.IsDisposed ) {
					BartDialog = new BartForm ( );
				}

				BartDialog.Show ( );
			} else {
				if ( PluginHost != null ) {
					PluginHost.MessageBox ( "Error", "Unable to launch bart manager because device is not in recovery mode",
						"The device must be in recovery mode in order to launch bart manager", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Error );
				} else {
					System.Windows.Forms.MessageBox.Show ( "Unable to launch bart manager because device is not in recovery mode", "Error", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Error );
				}
			}
		}

		/// <summary>
		/// Executes the specified plugin host.
		/// </summary>
		/// <param name="pluginHost">The plugin host.</param>
		/// <param name="args">The args.</param>
		public void Execute ( IPluginHost pluginHost, string[] args ) {
			Execute ( pluginHost, new DroidExplorer.Core.IO.LinuxDirectoryInfo ( "/" ), args );
		}

		/// <summary>
		/// Gets or sets the plugin host.
		/// </summary>
		/// <value>The plugin host.</value>
		public IPluginHost PluginHost { get; set; }

		/// <summary>
		/// Gets a value indicating whether this <see cref="Bart"/> is runnable.
		/// </summary>
		/// <value><c>true</c> if runnable; otherwise, <c>false</c>.</value>
		public bool Runnable {
			get { return true; }
		}

		/// <summary>
		/// Shows the configuration.
		/// </summary>
		public void ShowConfiguration ( ) {

		}

		/// <summary>
		/// Creates the tool strip menu item.
		/// </summary>
		/// <returns></returns>
		public ToolStripItem CreateToolStripMenuItem ( ) {
			return PluginHelper.CreateToolStripMenuItemForPlugin ( this );
		}

		/// <summary>
		/// Creates the tool strip button.
		/// </summary>
		/// <returns></returns>
		public ToolStripItem CreateToolStripButton ( ) {
			return PluginHelper.CreateToolStripButtonForPlugin ( this );
		}


		public int MinimumSDKToolsVersion {
			get { return 0; }
		}
		public int MinimumSDKPlatformToolsVersion {
			get { return 0; }
		}
		#endregion

		/// <summary>
		/// Gets or sets the bart dialog.
		/// </summary>
		/// <value>The bart dialog.</value>
		private BartForm BartDialog { get; set; }

		/// <summary>
		/// Gets a value indicating whether this has a signature match to the installed application.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if this instance is signature match; otherwise, <c>false</c>.
		/// </value>
		private bool IsSignatureMatch {
			get {
				try {
					string sig = GetSignature ( );
					string act = string.Format ( "{0}{1}", CommandRunner.Instance.DefaultDevice, SIGNATURE );
					this.LogInfo ( "Signature: {0}", sig );
					this.LogInfo ( "Expected: {0}", act );
					return string.Compare ( sig, act, false ) == 0;
				} catch ( Exception ex ) {
					this.LogWarn ( ex.Message, ex );
					return false;
				}
			}
		}

		#region IPaidPlugin Members
		bool mounted = false;
		/// <summary>
		/// Gets the signature.
		/// </summary>
		/// <returns></returns>
		public string GetSignature ( ) {
			if ( DeviceState == CommandRunner.DeviceState.Recovery && !mounted ) {
				CommandRunner.Instance.Mount ( "data" );
				CommandRunner.Instance.Mount ( "/system/sd" );
				CommandRunner.Instance.Mount ( "/sd-ext" );

			}
			try {
				return PluginHelper.GetPaidPluginApkSignature ( CommandRunner.Instance.DefaultDevice, BARTAPK );
			} catch ( Exception ) {
				try {
					return PluginHelper.GetPaidPluginApkSignature ( CommandRunner.Instance.DefaultDevice, BARTAPK1 );
				} catch ( Exception ) {
					this.LogInfo ( "Unable to located plugin in default location, checking sd-ext" );
					try {
						return PluginHelper.GetPaidPluginApkSignature ( CommandRunner.Instance.DefaultDevice, BARTALTAPK );
					} catch ( Exception ) {
						return PluginHelper.GetPaidPluginApkSignature ( CommandRunner.Instance.DefaultDevice, BARTALTAPK1 );
					}
				}
			}
		}

		#endregion
	}
}
