﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DroidExplorer.Core.Plugins;
using DroidExplorer.Plugins.UI;
using System.Drawing;
using System.IO;
using DroidExplorer.Core;
using System.ComponentModel;
using System.Windows.Forms;

namespace DroidExplorer.Plugins {
	/// <summary>
	/// 
	/// </summary>
	public class DesktopSms : BasePlugin {

		/// <summary>
		/// Initializes a new instance of the <see cref="DesktopSms" /> class.
		/// </summary>
		/// <param name="host">The host.</param>
		public DesktopSms ( IPluginHost host ) : base(host) {

		}

		#region IPlugin Members

		/// <summary>
		/// Gets the name.
		/// </summary>
		/// <value>
		/// The name.
		/// </value>
		public override string Name {
			get { return "DesktopSMS"; }
		}

		/// <summary>
		/// Gets the description.
		/// </summary>
		/// <value>
		/// The description.
		/// </value>
		public override string Description {
			get { return "DesktopSMS is an application for Android who allows you sending text messages (SMS) from your computer through your Android phone."; }
		}

		/// <summary>
		/// Gets the image.
		/// </summary>
		/// <value>
		/// The image.
		/// </value>
		public override System.Drawing.Image Image {
			get { return Properties.Resources.desktopSMS; }
		}

		/// <summary>
		/// Gets the text.
		/// </summary>
		/// <value>
		/// The text.
		/// </value>
		public override string Text {
			get { return "Desktop SMS"; }
		}

		/// <summary>
		/// Gets a value indicating whether [create tool button].
		/// </summary>
		/// <value>
		///   <c>true</c> if [create tool button]; otherwise, <c>false</c>.
		/// </value>
		public override bool CreateToolButton {
			get { return true; }
		}

		/// <summary>
		/// Executes the specified plugin host.
		/// </summary>
		/// <param name="pluginHost">The plugin host.</param>
		/// <param name="currentDirectory">The current directory.</param>
		/// <param name="args">The args.</param>
		public override void Execute ( IPluginHost pluginHost, DroidExplorer.Core.IO.LinuxDirectoryInfo currentDirectory, string[] args ) {
      try {
        string file = System.IO.Path.Combine ( CommandRunner.Instance.TempDataPath, "desktopsms.jnlp" );
        using ( FileStream fs = new FileStream ( file, FileMode.OpenOrCreate, FileAccess.Write, FileShare.Read ) ) {
          fs.Write ( Properties.Resources.desktopsmslauncher, 0, Properties.Resources.desktopsmslauncher.Length );
        }
        CommandRunner.Instance.LaunchProcessWindow ( file, string.Empty, true );
      } catch ( Exception ex ) {
        throw;
      }
		}

		#endregion

		#region IPluginExtendedInfo Members

		/// <summary>
		/// Gets the author.
		/// </summary>
		/// <value>
		/// The author.
		/// </value>
		public override string Author {
      get { return "Gaëtan Eleouet"; }
		}

		/// <summary>
		/// Gets the URL.
		/// </summary>
		/// <value>
		/// The URL.
		/// </value>
		public override string Url {
      get { return "http://sites.google.com/site/desktopsmsforandroid/en"; }
		}

		/// <summary>
		/// Gets the contact.
		/// </summary>
		/// <value>
		/// The contact.
		/// </value>
		public override string Contact {
			get { return string.Empty; }
		}


		#endregion

	}
}
