﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DroidExplorer.Core.Plugins;
using DroidExplorer.Core;
using System.IO;
using System.ComponentModel;
using System.Windows.Forms;

namespace DroidExplorer.Plugins {
	/// <summary>
	/// 
	/// </summary>
	public class Screencast : BasePlugin {
		/// <summary>
		/// Initializes a new instance of the <see cref="Screencast"/> class.
		/// </summary>
		/// <param name="pluginHost">The plugin host.</param>
		public Screencast ( IPluginHost host ) : base(host) {

		}

		#region IPlugin Members

		/// <summary>
		/// Gets the name.
		/// </summary>
		/// <value>The name.</value>
		public override string Name {
			get { return "AndroidScreencast"; }
		}

		/// <summary>
		/// Gets the description.
		/// </summary>
		/// <value>The description.</value>
		public override string Description {
			get { return "Control your android device witch is connected through USB using mouse and keyboard in the same way it can be done on an emulator."; }
		}

		/// <summary>
		/// Gets the image.
		/// </summary>
		/// <value>The image.</value>
		public override System.Drawing.Image Image {
			get { return Properties.Resources.screencast; }
		}

		/// <summary>
		/// Gets the text.
		/// </summary>
		/// <value>The text.</value>
		public override string Text {
			get { return "Screencast"; }
		}

		/// <summary>
		/// Gets a value indicating whether [create tool button].
		/// </summary>
		/// <value><c>true</c> if [create tool button]; otherwise, <c>false</c>.</value>
		public override bool CreateToolButton {
			get { return true; }
		}

		/// <summary>
		/// Executes the specified plugin host.
		/// </summary>
		/// <param name="pluginHost">The plugin host.</param>
		/// <param name="currentDirectory">The current directory.</param>
		public override void Execute ( IPluginHost pluginHost, DroidExplorer.Core.IO.LinuxDirectoryInfo currentDirectory, string[] args ) {
			try {
				CommandRunner.Instance.LaunchProcessWindow ( "javaws.exe", "http://androidscreencast.googlecode.com/svn/trunk/AndroidScreencast/dist/androidscreencast.jnlp", true );
			} catch ( Exception ) {
				try {
					string file = System.IO.Path.Combine ( CommandRunner.Instance.TempDataPath, "androidscreencast.jnlp" );
					using ( FileStream fs = new FileStream ( file, FileMode.OpenOrCreate, FileAccess.Write, FileShare.Read ) ) {
						fs.Write ( Properties.Resources.androidscreencast, 0, Properties.Resources.androidscreencast.Length );
					}
					CommandRunner.Instance.LaunchProcessWindow ( "javaws.exe", string.Format ( "\"{0}\"", file ), true );
				} catch ( Exception ) {
					throw;
				}
			}
		}
		#endregion

		#region IPluginExtendedInfo Members

		/// <summary>
		/// Gets the author.
		/// </summary>
		/// <value>The author.</value>
		public override string Author {
			get { return "Thiel Alexandre"; }
		}

		/// <summary>
		/// Gets the URL.
		/// </summary>
		/// <value>The URL.</value>
		public override string Url {
			get { return "http://code.google.com/p/androidscreencast/"; }
		}

		/// <summary>
		/// Gets the contact.
		/// </summary>
		/// <value>The contact.</value>
		public override string Contact {
			get { return "http://forum.xda-developers.com/member.php?u=1098962"; }
		}

		#endregion
	}
}
