﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DroidExplorer.Core;
using DroidExplorer.Core.Plugins;
using DroidExplorer.Plugins.UI;
using System.ComponentModel;
using System.Windows.Forms;

namespace DroidExplorer.Plugins {
	/// <summary>
	/// 
	/// </summary>
	public class SqliteExplorer : BasePlugin, IFileTypeHandler {
		private SqliteForm sqlEditor = null;

		/// <summary>
		/// Initializes a new instance of the <see cref="SqliteExplorer"/> class.
		/// </summary>
		/// <param name="host">The host.</param>
		public SqliteExplorer ( IPluginHost host ) : base(host) {
			if ( host != null ) {
				this.PluginHost.RegisterFileTypeIcon ( ".db", DroidExplorer.Resources.Images.database, DroidExplorer.Resources.Images.database32 );
				this.PluginHost.RegisterFileTypeIcon ( ".sqlite", DroidExplorer.Resources.Images.database, DroidExplorer.Resources.Images.database32 );
				this.PluginHost.RegisterFileTypeIcon ( ".database", DroidExplorer.Resources.Images.database, DroidExplorer.Resources.Images.database32 );

				this.PluginHost.RegisterFileTypeHandler ( ".db", this );
				this.PluginHost.RegisterFileTypeHandler ( ".sqlite", this );
				this.PluginHost.RegisterFileTypeHandler ( ".database", this );
			}
		}
		#region IPlugin Members

		/// <summary>
		/// Gets the name.
		/// </summary>
		/// <value>The name.</value>
		public override string Name {
			get { return "SqliteExplorer"; }
		}

		/// <summary>
		/// Gets the description.
		/// </summary>
		/// <value>The description.</value>
		public override string Description {
			get { return "a sql database browser"; }
		}

		/// <summary>
		/// Gets the image.
		/// </summary>
		/// <value>The image.</value>
		public override System.Drawing.Image Image {
			get { return DroidExplorer.Resources.Images.database; }
		}

		/// <summary>
		/// Gets the text.
		/// </summary>
		/// <value>The text.</value>
		public override string Text {
			get { return "Sqlite Explorer"; }
		}



		/// <summary>
		/// Executes the specified plugin host.
		/// </summary>
		/// <param name="pluginHost">The plugin host.</param>
		/// <param name="currentDirectory">The current directory.</param>
		/// <param name="args">The args.</param>
		public override void Execute ( IPluginHost pluginHost, DroidExplorer.Core.IO.LinuxDirectoryInfo currentDirectory, string[] args ) {
			if ( sqlEditor == null || sqlEditor.IsDisposed ) {
				sqlEditor = new SqliteForm ( );
			}
			this.LogDebug ( "Editor launch" );
			if ( pluginHost != null ) {
				sqlEditor.Show ( );
			} else {
				sqlEditor.ShowDialog ( );
			}
		}

		/// <summary>
		/// Gets a value indicating whether [create tool button].
		/// </summary>
		/// <value><c>true</c> if [create tool button]; otherwise, <c>false</c>.</value>
		public override bool CreateToolButton {
			get { return true; }
		}

		#endregion

		#region IPluginExtendedInfo Members

		/// <summary>
		/// Gets the author.
		/// </summary>
		/// <value>The author.</value>
		public override string Author {
			get { return "Ryan Conrad"; }
		}

		/// <summary>
		/// Gets the URL.
		/// </summary>
		/// <value>The URL.</value>
		public override string Url {
			get { return "http://de.codeplex.com/"; }
		}

		/// <summary>
		/// Gets the contact.
		/// </summary>
		/// <value>The contact.</value>
		public override string Contact {
			get { return string.Empty; }
		}

		#endregion

		#region IFileTypeHandler Members

		/// <summary>
		/// Opens the specified file.
		/// </summary>
		/// <param name="file">The file.</param>
		public void Open ( DroidExplorer.Core.IO.FileInfo file ) {
			if ( sqlEditor == null || sqlEditor.IsDisposed ) {
				sqlEditor = new SqliteForm ( file );
			} else {
				sqlEditor.OpenFile ( file.FullPath );
			}
			sqlEditor.Show ( );
		}

		#endregion



	}
}
