﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DroidExplorer.Core.Plugins;
using DroidExplorer.Core.UI;
using DroidExplorer.UI;
using DroidExplorer.Core;
using System.Threading;

namespace DroidExplorer.Plugins.UI {
  public partial class LogCatConsole : PluginForm, IShellProcess {
    public LogCatConsole ( ) {
      InitializeComponent ( );

      this.StickyWindow = new StickyWindow ( this );
      ConsoleOutput = new ConsoleWriter ( ref this.shell );
      ConsoleError = new ConsoleWriter ( ref this.shell );

      Run ( "logcat" );
    }

    private bool IsRunning { get { return this.BaseProcess != null; } }
    private ConsoleWriter ConsoleOutput { get; set; }
    private ConsoleWriter ConsoleError { get; set; }

    protected override void OnClosing ( CancelEventArgs e ) {
      if ( IsRunning ) {
        if ( !this.BaseProcess.HasExited ) {
          BaseProcess.Kill ( );
        }
        BaseProcess = null;
      }
      base.OnClosing ( e );
    }

    private void Run ( string command ) {
      CommandRunner.Instance.LaunchRedirectedShellWindow ( CommandRunner.Instance.DefaultDevice, command, this );
    }


    #region IShellProcess Members

    public System.IO.StreamReader OutputStream { get; private set; }

    public System.IO.StreamReader ErrorStream { get; private set; }

    public System.IO.StreamWriter InputStream { get; private set; }

    public System.Diagnostics.Process BaseProcess { get; private set; }

    public void SetBaseProcess ( System.Diagnostics.Process process ) {
      if ( process != null ) {
        BaseProcess = process;
      } else {
        BaseProcess = null;
        //WritePrompt ( );
      }
    }

    public void SetOutputStream ( System.IO.StreamReader reader ) {
      OutputStream = reader;
      new Thread ( new ThreadStart ( delegate {
        while ( IsRunning ) {
          while ( !OutputStream.EndOfStream ) {
            string line = OutputStream.ReadLine ( );
            if ( !string.IsNullOrEmpty ( line ) ) {
              ConsoleOutput.WriteLine ( line );
            }
          }
        }
      } ) ).Start ( );
    }

    public void SetErrorStream ( System.IO.StreamReader reader ) {
      ErrorStream = reader;
    }

    public void SetInputStream ( System.IO.StreamWriter writer ) {
      InputStream = writer;
    }


    #endregion

    private void saveToolStripButton_Click ( object sender, EventArgs e ) {
      System.Windows.Forms.SaveFileDialog sfd = new System.Windows.Forms.SaveFileDialog ( );
      sfd.Title = "Save logcat";
      sfd.Filter = "Log file|*.log";
      sfd.FilterIndex = 0;
      if ( sfd.ShowDialog ( this ) == DialogResult.OK ) {
        this.shell.SaveFile ( sfd.FileName, RichTextBoxStreamType.PlainText );
      }
    }

    private void clearToolStripButton_Click ( object sender, EventArgs e ) {
      this.shell.Clear ( );
    }
  }
}
