﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Drawing.Imaging;
using System.Threading;
using System.Net;
using DroidExplorer.Core.Adb;
using DroidExplorer.Core.IO;
using DroidExplorer.Core;
using DroidExplorer.Core.UI;
using DroidExplorer.UI;

namespace DroidExplorer.Plugins.UI {
	/// <summary>
	/// 
	/// </summary>
	public partial class ScreenShotForm : PluginForm {
		private delegate void SetPictureBoxImageDelegate ( PictureBox pb, Image img );
		private delegate void SetToolStripItemEnabledDelegate ( ToolStripItem tsi, bool enabled );
		private delegate Image GetPictureBoxImageDelegate ( PictureBox pb );
		private delegate void SetIntegerDelegate ( int value );
		private delegate int GetIntegerDelegate ( );
		private delegate void GenericDelegate ( );
		private delegate bool GetToolStripButtonCheckedDelegate ( ToolStripButton tsi );
		private delegate void SetControlBooleanDelegate ( bool value );
		/// <summary>
		/// Initializes a new instance of the <see cref="ScreenShotForm"/> class.
		/// </summary>
		public ScreenShotForm ( ) {
			InitializeComponent ( );
			this.StickyWindow = new StickyWindow ( this );
			RefreshImage ( );
		}

		/// <summary>
		/// Handles the Click event of the refreshToolStripButton control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void refreshToolStripButton_Click ( object sender, EventArgs e ) {
			RefreshImage ( );
		}

		/// <summary>
		/// Gets the image from file.
		/// </summary>
		/// <param name="fb0">The FB0.</param>
		/// <returns></returns>
		private Image GetImageFromFile ( string fb0 ) {
			try {
				Image img = Rgb565.ToImage ( fb0 );
				File.Delete ( fb0 );

				return img;
			} catch ( Exception ex ) {
				Console.WriteLine ( ex );
				return null;
			}
		}


		/// <summary>
		/// Gets the image from raw image.
		/// </summary>
		/// <param name="ri">The ri.</param>
		/// <returns></returns>
		private Image GetImageFromRawImage ( RawImage ri ) {
			try {
				this.LogDebug ( "Image Size: {0}", ri.Size.ToString ( ) );
				if ( ri.Bpp == 16 ) {
					return ri.ToImage ( PixelFormat.Format16bppRgb565 );
				} else if ( ri.Bpp == 32 ) {
					return ri.ToImage ( PixelFormat.Format32bppArgb );
				} else {
					return Rgb565.ToImage ( ri.Data );
				}
			} catch ( Exception ex ) {
				Console.WriteLine ( ex );
				return null;
			}
		}

		/// <summary>
		/// Refreshes the image.
		/// </summary>
		private void RefreshImage ( ) {
			if ( !ThreadRunning ) {
				new Thread ( new ThreadStart ( delegate {
					ThreadRunning = true;
					if ( this.InvokeRequired ) {
						this.Invoke ( new SetToolStripItemEnabledDelegate ( SetToolStripItemEnabled ), copyToolStripButton, false );
						this.Invoke ( new SetToolStripItemEnabledDelegate ( SetToolStripItemEnabled ), saveToolStripButton, false );
						this.Invoke ( new SetToolStripItemEnabledDelegate ( SetToolStripItemEnabled ), PLModeToolStripButton, false );
						this.Invoke ( new SetToolStripItemEnabledDelegate ( SetToolStripItemEnabled ), refreshToolStripButton, false );
					} else {
						SetToolStripItemEnabled ( copyToolStripButton, false );
						SetToolStripItemEnabled ( saveToolStripButton, false );
						SetToolStripItemEnabled ( PLModeToolStripButton, false );
						SetToolStripItemEnabled ( refreshToolStripButton, false );
					}

					Image img = GetScreenShot ( );

					if ( GetIsLandscapeModeChecked ( ) ) {
						FlipImage ( img );
					}


					if ( this.InvokeRequired ) {
						this.Invoke ( new SetPictureBoxImageDelegate ( SetPictureBoxImage ), screen, null );
						this.Invoke ( new SetPictureBoxImageDelegate ( SetPictureBoxImage ), screen, img );
					} else {
						SetPictureBoxImage ( this.screen, null );
						SetPictureBoxImage ( this.screen, img );
					}

					ResizeFromImage ( img );

					if ( this.InvokeRequired ) {
						this.Invoke ( new SetToolStripItemEnabledDelegate ( SetToolStripItemEnabled ), copyToolStripButton, true );
						this.Invoke ( new SetToolStripItemEnabledDelegate ( SetToolStripItemEnabled ), saveToolStripButton, true );
						this.Invoke ( new SetToolStripItemEnabledDelegate ( SetToolStripItemEnabled ), PLModeToolStripButton, true );
						this.Invoke ( new SetToolStripItemEnabledDelegate ( SetToolStripItemEnabled ), refreshToolStripButton, true );

					} else {
						SetToolStripItemEnabled ( copyToolStripButton, true );
						SetToolStripItemEnabled ( saveToolStripButton, true );
						SetToolStripItemEnabled ( PLModeToolStripButton, true );
						SetToolStripItemEnabled ( refreshToolStripButton, true );
					}

					ThreadRunning = false;
				} ) ).Start ( );
			}
		}


		/// <summary>
		/// Gets the screen shot.
		/// </summary>
		/// <returns></returns>
		private Image GetScreenShot ( ) {
			Device d = new Device ( CommandRunner.Instance.DefaultDevice, DeviceState.Online );
			RawImage ri = AdbHelper.Instance.GetFrameBuffer ( AndroidDebugBridge.SocketAddress, d );
			return GetImageFromRawImage ( ri );
		}


		/// <summary>
		/// Resizes from image.
		/// </summary>
		/// <param name="img">The img.</param>
		private void ResizeFromImage ( Image img ) {
			if ( img == null ) {
				img = new Bitmap ( 1, 1 );
			}
			if ( this.InvokeRequired ) {
				int b = (int)this.Invoke ( new GetIntegerDelegate ( this.GetToolStripBottom ) );
				this.Invoke ( new SetIntegerDelegate ( this.SetWidth ), img.Width + (SystemInformation.BorderSize.Width*2) + 20 );
				this.Invoke ( new SetIntegerDelegate ( this.SetHeight ), b + img.Height + SystemInformation.BorderSize.Height + 20 + SystemInformation.CaptionHeight );
			} else {
				this.SetWidth ( img.Width + (SystemInformation.BorderSize.Width*2) + 20 );
				this.SetHeight ( GetToolStripBottom ( ) + img.Height + SystemInformation.BorderSize.Height + 20 + SystemInformation.CaptionHeight );
			}
		}

		/// <summary>
		/// Flips the image.
		/// </summary>
		/// <param name="img">The img.</param>
		/// <returns></returns>
		private Image FlipImage ( Image img ) {
			if ( GetIsLandscapeModeChecked ( ) ) {
				img.RotateFlip ( RotateFlipType.Rotate270FlipNone );
			} else {
				img.RotateFlip ( RotateFlipType.Rotate90FlipNone );
			}
			return img;
		}


		/// <summary>
		/// Handles the Click event of the saveToolStripButton control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void saveToolStripButton_Click ( object sender, EventArgs e ) {
			System.Windows.Forms.SaveFileDialog sfd = new System.Windows.Forms.SaveFileDialog ( );
			sfd.Filter = "Portable Network Graphic (PNG)|*.png";
			sfd.FilterIndex = 0;
			sfd.FileName = "screenshot.png";
			if ( sfd.ShowDialog ( this ) == DialogResult.OK ) {
				this.screen.Image.Save ( sfd.FileName, ImageFormat.Png );
			}
		}

		/// <summary>
		/// Gets a value indicating whether this instance is portrait mode.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if this instance is portrait mode; otherwise, <c>false</c>.
		/// </value>
		public bool IsPortraitMode {
			get {
				if ( this.InvokeRequired ) {
					return ( (int)this.Invoke ( new GetIntegerDelegate ( this.GetWidth ) ) ) < (int)this.Invoke ( new GetIntegerDelegate ( this.GetHeight ) );
				} else {
					return this.Width < this.Height;
				}
			}
		}
		/// <summary>
		/// Gets or sets a value indicating whether [thread running].
		/// </summary>
		/// <value><c>true</c> if [thread running]; otherwise, <c>false</c>.</value>
		private bool ThreadRunning { get; set; }

		/// <summary>
		/// Gets the is landscape mode checked.
		/// </summary>
		/// <returns></returns>
		private bool GetIsLandscapeModeChecked ( ) {
			if ( this.InvokeRequired ) {
				return (bool)this.Invoke ( new GetToolStripButtonCheckedDelegate ( this.GetToolStripButtonChecked ), PLModeToolStripButton );
			} else {
				return this.PLModeToolStripButton.Checked;
			}
		}

		/// <summary>
		/// Gets the tool strip button checked.
		/// </summary>
		/// <param name="tsi">The tsi.</param>
		/// <returns></returns>
		private bool GetToolStripButtonChecked ( ToolStripButton tsi ) {
			return tsi.Checked;
		}

		/// <summary>
		/// Handles the Click event of the copyToolStripButton control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void copyToolStripButton_Click ( object sender, EventArgs e ) {
			Clipboard.SetImage ( this.screen.Image );
		}

		/// <summary>
		/// Handles the CheckedChanged event of the PLModeToolStripButton control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void PLModeToolStripButton_CheckedChanged ( object sender, EventArgs e ) {
			Image baseImage = this.screen.Image.Clone ( ) as Image;
			this.SetPictureBoxImage ( screen, null );
			FlipImage ( baseImage );
			ResizeFromImage ( baseImage );
			this.SetPictureBoxImage ( screen, baseImage );
		}


		/// <summary>
		/// Handles the MouseUp event of the pictureBox control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
		private void pictureBox_MouseUp ( object sender, MouseEventArgs e ) {
			if ( e.Button == MouseButtons.Right ) {
				PLModeToolStripButton.Checked = !PLModeToolStripButton.Checked;
			}
		}

		/// <summary>
		/// Handles the Click event of the PLModeToolStripButton control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void PLModeToolStripButton_Click ( object sender, EventArgs e ) {
			PLModeToolStripButton.Checked = !PLModeToolStripButton.Checked;
		}

		/// <summary>
		/// Sets the picture box image.
		/// </summary>
		/// <param name="pb">The pb.</param>
		/// <param name="img">The img.</param>
		private void SetPictureBoxImage ( PictureBox pb, Image img ) {
			pb.Image = img;
		}

		/// <summary>
		/// Gets the picture box image.
		/// </summary>
		/// <param name="pb">The pb.</param>
		/// <returns></returns>
		private Image GetPictureBoxImage ( PictureBox pb ) {
			return pb.Image;
		}

		/// <summary>
		/// Sets the tool strip item enabled.
		/// </summary>
		/// <param name="tsi">The tsi.</param>
		/// <param name="enabled">if set to <c>true</c> [enabled].</param>
		private void SetToolStripItemEnabled ( ToolStripItem tsi, bool enabled ) {
			tsi.Enabled = enabled;
		}

		/// <summary>
		/// Gets the width.
		/// </summary>
		/// <returns></returns>
		private int GetWidth ( ) {
			return this.Width;
		}

		/// <summary>
		/// Gets the height.
		/// </summary>
		/// <returns></returns>
		private int GetHeight ( ) {
			return this.Height;
		}

		/// <summary>
		/// Sets the width.
		/// </summary>
		/// <param name="value">The value.</param>
		private void SetWidth ( int value ) {
			this.Width = value;
		}

		/// <summary>
		/// Sets the height.
		/// </summary>
		/// <param name="value">The value.</param>
		private void SetHeight ( int value ) {
			this.Height = value;
		}

		/// <summary>
		/// Gets the tool strip bottom.
		/// </summary>
		/// <returns></returns>
		private int GetToolStripBottom ( ) {
			return this.toolStrip1.Bottom;
		}


	}

}
