﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using DroidExplorer.Core;
using System.Data.SQLite;
using DroidExplorer.Core.IO;
using DroidExplorer.Plugins.Data;
using DroidExplorer.Core.UI;

namespace DroidExplorer.Plugins.UI {
	/// <summary>
	/// 
	/// </summary>
	public partial class SqliteForm : Form {
		SQLiteConnection connection;
		System.IO.FileInfo databaseFile = null;
		string originalFile = string.Empty;

		/// <summary>
		/// Initializes a new instance of the <see cref="SqliteForm"/> class.
		/// </summary>
		public SqliteForm ( )
			: base ( ) {
			InitializeComponent ( );
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="SqliteForm"/> class.
		/// </summary>
		/// <param name="file">The file.</param>
		public SqliteForm ( DroidExplorer.Core.IO.FileInfo file )
			: this ( ) {
			OpenFile ( file.FullPath );
		}

		/// <summary>
		/// Handles the Click event of the openToolStripButton control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void openToolStripButton_Click ( object sender, EventArgs e ) {
			DroidExplorer.Core.UI.OpenFileDialog ofd = new DroidExplorer.Core.UI.OpenFileDialog ( );
			ofd.CustomPlaces.Add ( new DroidExplorer.Core.UI.FileDialogCustomPlace ( "/data/data/", "data", DroidExplorer.Resources.Images.database32, delegate ( object s, EventArgs ea ) {
				ofd.Navigate ( new LinuxDirectoryInfo ( "/data/data/" ) );
			} ) );

			ofd.Filter = "Database Files|*.sqlite;*.db;*.database|All Files|*";
			ofd.FilterIndex = 0;
			ofd.Title = "Open Database";
			ofd.Multiselect = false;
			if ( ofd.ShowDialog ( this ) == DialogResult.OK ) {
				OpenFile ( ofd.FileName );
			}
		}

		/// <summary>
		/// Opens the file.
		/// </summary>
		/// <param name="file">The file.</param>
		internal void OpenFile ( string file ) {
			originalFile = file;
			databaseFile = CommandRunner.Instance.PullFile ( originalFile );
			if ( connection != null ) {
				connection.Close ( );
			}
			connection = new SQLiteConnection ( string.Format ( CultureInfo.InvariantCulture, "Data Source={0};Version=3;", databaseFile.FullName ) );
			connection.Open ( );
			BuildDatabaseTree ( );
		}

		/// <summary>
		/// Builds the database tree.
		/// </summary>
		private void BuildDatabaseTree ( ) {
			this.database.Nodes.Clear ( );
			TreeNode dbs = new TreeNode ( "Tables" );
			GetDatabasesForTree ( Properties.Resources.SqliteGetTables, dbs.Nodes );
			this.database.Nodes.Add ( dbs );

			TreeNode views = new TreeNode ( "Views" );
			GetDatabasesForTree ( Properties.Resources.SqliteGetViews, views.Nodes );
			this.database.Nodes.Add ( views );

			TreeNode triggers = new TreeNode ( "Triggers" );
			GetDatabasesForTree ( Properties.Resources.SqliteGetTriggers, triggers.Nodes );
			this.database.Nodes.Add ( triggers );

			TreeNode indexes = new TreeNode ( "Indexes" );
			GetDatabasesForTree ( Properties.Resources.SqliteGetIndexes, indexes.Nodes );
			this.database.Nodes.Add ( indexes );
		}

		/// <summary>
		/// Gets the databases for tree.
		/// </summary>
		/// <param name="sql">The SQL.</param>
		/// <param name="parent">The parent.</param>
		private void GetDatabasesForTree ( String sql, TreeNodeCollection parent ) {
			SQLiteCommand cmd = new SQLiteCommand ( sql, connection );
			using ( SQLiteDataReader reader = cmd.ExecuteReader ( ) ) {
				while ( reader.Read ( ) ) {
					parent.Add ( reader.GetString ( 1 ) );
				}
			}

			foreach ( TreeNode item in parent ) {
				GetDatabaseColumnsForTree ( string.Format ( CultureInfo.InvariantCulture, SqliteHelper.PragmaTableInfo, item.Text ), item.Nodes );
			}
		}

		/// <summary>
		/// Gets the database columns for tree.
		/// </summary>
		/// <param name="sql">The SQL.</param>
		/// <param name="parent">The parent.</param>
		private void GetDatabaseColumnsForTree ( String sql, TreeNodeCollection parent ) {
			SQLiteCommand cmd = new SQLiteCommand ( sql, connection );
			using ( SQLiteDataReader reader = cmd.ExecuteReader ( ) ) {
				while ( reader.Read ( ) ) {
					parent.Add ( reader.GetString ( 1 ) );
				}
			}
		}

		/// <summary>
		/// Raises the <see cref="E:System.Windows.Forms.Form.Closing"/> event.
		/// </summary>
		/// <param name="e">A <see cref="T:System.ComponentModel.CancelEventArgs"/> that contains the event data.</param>
		protected override void OnClosing ( CancelEventArgs e ) {
			if ( connection != null ) {
				connection.Close ( );
			}
			base.OnClosing ( e );
		}

		/// <summary>
		/// Handles the Click event of the saveToolStripButton control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void saveToolStripButton_Click ( object sender, EventArgs e ) {
			if ( databaseFile != null ) {
				DroidExplorer.Core.UI.SaveFileDialog sfd = new DroidExplorer.Core.UI.SaveFileDialog ( );
				sfd.CustomPlaces.Add ( new DroidExplorer.Core.UI.FileDialogCustomPlace ( "/data/data/", "Device Data", DroidExplorer.Resources.Images.database32, delegate ( object s, EventArgs ea ) {
					sfd.Navigate ( new LinuxDirectoryInfo ( "/data/data/" ) );
				} ) );
				sfd.Title = "Save Database";
				sfd.Filter = "Database Files|*.sqlite;*.db;*.database|All Files|*";
				sfd.FilterIndex = 0;
				sfd.InitialDirectory = DroidExplorer.Core.IO.Path.GetPathWithoutFile ( this.originalFile );
				sfd.FileName = DroidExplorer.Core.IO.Path.GetFileName ( this.originalFile );
				if ( sfd.ShowDialog ( this ) == DialogResult.OK ) {
					CommandRunner.Instance.PushFile ( databaseFile.FullName, sfd.FileName );
				}
			}
		}

		/// <summary>
		/// Handles the Click event of the runSql control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void runSql_Click ( object sender, EventArgs e ) {
			SQLiteCommand cmd = new SQLiteCommand ( this.sql.Text, connection );
			DataSet ds = new DataSet ( );
			using ( SQLiteDataAdapter adapter = new SQLiteDataAdapter ( cmd ) ) {
				try {
					adapter.Fill ( ds );
					if ( ds.Tables.Count > 0 ) {
						this.results.DataSource = ds.Tables[ 0 ];
					} else {
						this.results.DataSource = null;
					}
				} catch ( SQLiteException sex ) {
					DataTable dt = new DataTable ( "error" );
					dt.Columns.Add ( new DataColumn ( "Message", typeof ( string ) ) );
					dt.Columns.Add ( new DataColumn ( "Source", typeof ( string ) ) );
					dt.Columns.Add ( new DataColumn ( "ErrorCode", typeof ( int ) ) );
					dt.Columns.Add ( new DataColumn ( "StackTrace", typeof ( string ) ) );
					dt.Rows.Add ( sex.Message, sex.Source, sex.ErrorCode, sex.StackTrace );
					this.results.DataSource = dt;
				}
			}
		}
	}
}
