﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Deployment.WindowsInstaller;
using System.IO;
using System.Collections;
using Microsoft.Win32;

namespace DroidExplorer.WixCA {
	public class AndroidSdkCustomActions {
		[CustomAction]
		public ActionResult VerifyAndroidSdkPath ( Session session ) {
			string path = session.CustomActionData["ANDROIDSDKPATH"];
			DirectoryInfo sdk = new DirectoryInfo ( path );
			if ( !sdk.Exists ) {
				return ActionResult.Failure;
			} else {
				DirectoryInfo tools = new DirectoryInfo ( Path.Combine ( sdk.FullName, "tools" ) );
				if ( !tools.Exists ) {
					return ActionResult.Failure;
				}

				FileInfo adb = new FileInfo ( Path.Combine ( tools.FullName, "adb.exe" ) );
				if ( !adb.Exists ) {
					return ActionResult.Failure;
				}					
				return GetLatestSdkPlatform(session);
			}
		}

		[CustomAction]
		public ActionResult GetLatestSdkPlatform ( Session session ) {
			string[] propValue = session.CustomActionData["ANDROIDSDKPATH"].Split(';');
			string path = propValue[0];
			string regPath = propValue[1];
			DirectoryInfo sdk = new DirectoryInfo ( path );
			if ( !sdk.Exists ) {
				return ActionResult.Failure;
			} else {
				DirectoryInfo platforms = new DirectoryInfo ( Path.Combine ( path, "platforms" ) );
				String platformBaseName = "android-";
				List<String> versions = new List<String> ( );
				foreach ( var item in platforms.GetDirectories(platformBaseName + "*") ) {
					try {
						String version = item.Name.Substring ( platformBaseName.Length );
						versions.Add ( version );
					} catch ( Exception ) {
						throw;
					}
				}

				if ( versions.Count == 0 ) {
					return ActionResult.Failure;
				}

				versions.Sort ( );


				using ( RegistryKey key = Registry.LocalMachine.CreateSubKey ( regPath ) ) {
					if ( key != null ) {
						key.SetValue ( "Platform", versions[versions.Count - 1] );
						return ActionResult.Success;
					}
				}
				return ActionResult.Failure;
			}
		}

	}
}
